@@ -1,14 +1,15 @@
 package RDF::Closure::AxiomaticTriples;
 
 use 5.008;
-use common::sense;
+use strict;
+use utf8;
 
 use RDF::Trine qw[statement];
 use RDF::Trine::Namespace qw[RDF RDFS OWL XSD];
 
 use base qw[Exporter];
 
-our $VERSION = '0.000_03';
+our $VERSION = '0.000_04';
 
 our @EXPORT = qw[];
 our @EXPORT_OK = qw[
@@ -680,7 +681,7 @@ Toby Inkster E<lt>tobyink@cpan.orgE<gt>.
 
 Copyright 2008-2011 Ivan Herman
 
-Copyright 2011 Toby Inkster
+Copyright 2011-2012 Toby Inkster
 
 This library is free software; you can redistribute it and/or modify it
 under any of the following licences:
@@ -698,5 +699,13 @@ or (at your option) any later version.
 
 =back
 
+
+=head1 DISCLAIMER OF WARRANTIES
+
+THIS PACKAGE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED
+WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
+MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
+
+
 =cut
 
