@@ -1,12 +1,14 @@
 package RDF::Closure::Rule::Core;
 
 use 5.008;
-use common::sense;
+use strict;
+use utf8;
 
 use Error qw[:try];
 use RDF::Trine;
+use Time::HiRes qw[time];
 
-our $VERSION = '0.000_03';
+our $VERSION = '0.000_04';
 
 sub name
 {
@@ -27,4 +29,19 @@ sub apply_to_closure
 	throw Error "This method should not be called directly; subclasses should override it.";
 }
 
+sub pre_atc
+{
+	my ($self) = @_;
+	$self->debug('BEGIN');
+	$self->{start_time} = time();
+}
+
+sub post_atc
+{
+	my ($self) = @_;
+	$self->debug(sprintf("END %03.03f seconds", (time() - $self->{start_time})));
+	delete $self->{start_time};
+}
+
 1;
+
